using System.Collections.Generic;

using AntMe.English;


//Insert a slashdot behind AntMe.Player followed by your name (please don't use blanks).
//It could look like this: "AntMe.Player.NickBoesl".
namespace AntMe.Player.dancho
{
    //Using the 'player'-attribute you can specify the name of your ant colony and the 
    //first and last name of the player. In order for the game to find your ant colony you
    //have to name your colony; inserting your name 
    [Player(
        ColonyName = "dancho",
        FirstName = "Yordan",
        LastName = "Chaparov"
        )]

    //The 'caste'-attribute makes it possible to change the properties of your ants. You can make 
    //them faster or give them more energy etc. But be careful: Only '-1', '0', '1' and '2' can be
    //be used and the sum of all properties of one caste has to be '0'!
    //To use this attribute you have to name the caste fist. After that you can return the name of that
    //type by using 'ChooseType'.
    //For further information how to use the ant-types see chapter 6 of the tutorial. 
    [Caste(
        Name = "Default",
        SpeedModificator = 0,
        LoadModificator = 0,
        AttackModificator = 0,
        EnergyModificator = 0,
        RangeModificator = 0,
        ViewRangeModificator = 0,
        RotationSpeedModificator = 0
        )]
    [Caste(
        Name = "Gatherer",
        SpeedModificator = 0,
        LoadModificator = 1,
        AttackModificator = 0,
        EnergyModificator = -1,
        RangeModificator = 0,
        ViewRangeModificator = 0,
        RotationSpeedModificator = 0
        )]
    [Caste(
        Name = "Fighter",
        SpeedModificator = 0,
        LoadModificator = -1,
        AttackModificator = 1,
        EnergyModificator = 0,
        RangeModificator = 0,
        ViewRangeModificator = 0,
        RotationSpeedModificator = 0
        )]
    [Caste(
        Name = "Scout",
        SpeedModificator = 2,
        LoadModificator = -1,
        AttackModificator = -1,
        EnergyModificator = -1,
        RangeModificator = -1,
        ViewRangeModificator = 1,
        RotationSpeedModificator = 1
        )]

    public class Template : BaseAnt
    {
        private int sawfruit = 0;
        private int sawsugar = 0;
        private int sawbug = 0;
        private int aggro = 0;
//        private int cnt = 0;

        #region Castes

        /// <summary>
        /// Chooses a specific caste.
        /// </summary>
        /// <param name="typeCount">List of available ants from caste</param>
        /// <returns>Name of chosen caste</returns>
        public override string ChooseType(Dictionary<string, int> typeCount)
        {
            if (typeCount["Scout"] < 5)
            {
                return "Scout";
            }
            else if (typeCount["Fighter"] < 8)
            {
                return "Fighter";
            }
            else if (typeCount["Gatherer"] < 70)
            {
                return "Gatherer";
            }
            return "Fighter";
        }

        #endregion

        #region Movement

        
        /// <summary>
        /// Is called periodically everytime the ant does not know where to go.
        /// </summary>
        public override void Waits() 
        {
            GoAhead();
        }

        /// <summary>
        /// Is called once when the ant has exceeded a third of it's maximum range.
        /// </summary>
        public override void BecomesTired() 
        { 
        }

        #endregion

        #region Food

        /// <summary>
        /// Is called periodically everytime the ant spots at least one heap of sugar.
        /// </summary>
        /// <param name="sugar">The closest heap of sugar.</param>
        public override void Spots(Sugar sugar)
        {
            if ( (Caste == "Gatherer") && (!(Target is Fruit)) )
            {
                if (CurrentLoad == 0 )
                    GoToTarget(sugar);
            }
            if (Caste == "Scout")
            {
                MakeMark(590, 400);
//                TurnByDegrees(60);
            }
        }

        /// <summary>
        /// Is called periodically everytime the ant spots at least one piece of fruit.
        /// </summary>
        /// <param name="fruit">The closest piece of fruit.</param>
        public override void Spots(Fruit fruit) 
        {
            if (Caste == "Gatherer")
            {
                if (CurrentLoad == 0 && NeedsCarrier(fruit)&&(!(Target is Ant)) )
                    GoToTarget(fruit);
            }
            if (Caste == "Scout")
            {
                if (NeedsCarrier(fruit))
                {
                    MakeMark(600, 400);
                }
            }
        }

        /// <summary>
        /// Is called once when a sugar heap is the ant's target and it reaches the sugar.
        /// </summary>
        /// <param name="sugar">The heap of sugar.</param>
        public override void TargetReached(Sugar sugar) 
        {
            if ( Caste == "Gatherer" )
            {
                MakeMark(590, 400);
                Take(sugar);
                GoBackToAnthill();
                sawsugar = 0;
                sawfruit = 0;
            }
            else if (Caste == "Scout")
            {
                MakeMark(590, 400);
//                TurnByDegrees(60);
            }
        }

        /// <summary>
        /// Is called once when a piece of fruit is the ant's target and it reaches the fruit.
        /// </summary>
        /// <param name="fruit">The piece of fruit.</param>
        public override void TargetReached(Fruit fruit) 
        {
            if (Caste == "Gatherer")
            {
                if (NeedsCarrier(fruit))
                {
                    Take(fruit);
                    GoBackToAnthill();
                    sawfruit = 0;
                    sawsugar = 0;
                }
                else
                {
                    aggro = 1;
                }
            }
            else if (Caste == "Scout")
            {
                MakeMark(600, 400);
//                TurnByDegrees(60);
            }
        }

        #endregion

        #region Communication

        /// <summary>
        /// Is called once when the ant 'smells' a marker from its own ant colleagues.
        /// A marker can only be 'smelled' once and will never be smelled again.
        /// </summary>
        /// <param name="marker">Closest new marker.</param>
        public override void SmellsFriend(Marker marker) 
        {
            if (Caste == "Scout") return;
            //cnt++;
            //if (cnt == 10) cnt = 0;
            //if (cnt != 1) return;
            if ( ( Target == null) && (marker.Information <= 560 ) && ( Caste == "Gatherer" ) )
            {
                TurnToDirection(marker.Information);
                GoAhead();
                sawsugar = 1;
            }
            else if ((Target == null || Target is Sugar || (Target is Anthill && CurrentLoad > 0 && CarringFruit==null)) && (marker.Information == 600))
            {
                if (Caste == "Gatherer" || ( Caste == "Fighter" && sawbug == 0 ) )
                {
                    sawfruit = 1;
                    GoToTarget(marker);
                }
            }
            else if (marker.Information == 599 && CurrentLoad == 0)
            {
                if (Caste == "Fighter")
                {
                    sawbug = 1;
                    GoToTarget(marker);
                }
                else if (Caste == "Gatherer" && Target == null)
                {
                    aggro = 1;
                    GoToTarget(marker);
                }
            }
            else if (marker.Information == 598 && CurrentLoad == 0)
            {
                if (Caste == "Fighter")
                {
                    sawbug = 1;
                    GoToTarget(marker);
                }
                else if (Caste == "Gatherer" && Target == null)
                {
                    GoToTarget(marker);
                }
            }
            else if (marker.Information == 580 && Caste == "Fighter" && sawbug == 0)
            {
                if (Target == null)
                    GoToTarget(marker);
            }
            else if (marker.Information == 590 && CurrentLoad == 0 && Target == null && sawfruit == 0 && sawsugar == 0)
            {
                if (Caste == "Gatherer")
                {
                    sawsugar = 1;
                    GoToTarget(marker);
                }
            }
        }

        /// <summary>
        /// Is called repeatedly each time at least one other ant from the same colony is spotted.
        /// </summary>
        /// <param name="ant">The closest friendly ant.</param>
        public override void SpotsFriend(Ant ant) 
        {
        }

        /// <summary>
        /// Is called, when the ant spots a friendly/allied ant from another team.
        /// </summary>
        /// <param name="ant"></param>
        public override void SpotsPartner(Ant ant) 
        { 
        }

        #endregion

        #region Fight

        /// <summary>
        /// Is called repeatedly each time the ant spots at least one bug.
        /// </summary>
        /// <param name="bug">The closest bug.</param>
        public override void SpotsEnemy(Bug bug) 
        {
            if ((Caste == "Fighter")||(Caste=="Gatherer"&&(Target==null||aggro==1)))
            {
                if ((FriendlyAntsInViewrange >= 4) || (aggro == 1))
                {
                    if ((DistanceToAnthill > 60) || (aggro == 1)|| (Caste == "Fighter"))
                    {
                        Drop();
                        Attack(bug);
                        sawbug = 0;
                    }
                }
                else
                {
                    MakeMark(598, 200);
                    GoBackToAnthill();
                    sawbug = 0;
                }
            }
            else if (Caste == "Scout")
            {
                MakeMark(599, 200);
            }
        }

        /// <summary>
        /// Is called repeatedly each time the ant spots at least one ant of another colony.
        /// </summary>
        /// <param name="ant">The closest inimical ant.</param>
        public override void SpotsEnemy(Ant ant) 
        {
            if (Caste == "Fighter")
            {
                Attack(ant);
            }
            else if (Caste == "Scout")
            {
                MakeMark(580, 150);
            }
            else if (Caste == "Gatherer")
            {
                if (ant.CurrentLoad > 0)
                {
                    Drop();
                    Attack(ant);
                }
            }
        }

        /// <summary>
        /// Is called repeatedly each time the ant is attacked by a bug.
        /// </summary>
        /// <param name="bug">Closest attacking bug.</param>
        public override void UnderAttack(Bug bug) 
        {
            if (CurrentEnergy < MaximumEnergy / 2)
                GoAhead();
        }

        /// <summary>
        /// Is called repeatedly each time the ant is attacked by another ant.
        /// </summary>
        /// <param name="ant">The closest inimical attacking ant.</param>
        public override void UnderAttack(Ant ant) 
        {
            Drop();
            Attack(ant);
        }

        #endregion

        #region Misc

        /// <summary>
        /// Is called when the ant has just died.
        /// </summary>
        /// <param name="kindOfDeath">The reason for the ant's death.</param>
        public override void HasDied(KindOfDeath kindOfDeath) 
        { 
        }

        /// <summary>
        /// Is called each simulation step and not inflicted by any other external influence.
        /// </summary>
        public override void Tick() 
        {
            if (IsTired == true)
            {
                GoBackToAnthill();
                return;
            }
            if (DistanceToAnthill < 50) aggro = 0;
            if (CurrentLoad > 0)
            {
                GoBackToAnthill();
            }
            if (CurrentEnergy < MaximumEnergy / 3 && Target == null)
            {
                GoBackToAnthill();
            }
            else if (Target is Anthill && CurrentLoad > 0 && CarringFruit == null)
            {
                MakeMark(Direction + 180);
                sawfruit = 0;
                sawsugar = 0;
            }
            else if (Target is Anthill && CurrentLoad > 0 && CarringFruit != null)
            {
                if (NeedsCarrier(CarringFruit))
                {
                    MakeMark(600, 300);
                }
                sawfruit = 0;
                sawsugar = 0;
            }
            else if ( (Target is Bug) && (CurrentEnergy*FriendlyAntsInViewrange <= 1000) && ( DistanceToTarget <= 50 ) )
            {
                MakeMark(599, 2000);
                sawfruit = 0;
                sawsugar = 0;
            }
            /*else if (Target is Ant)
            {
                MakeMark(580, 200);
                sawfruit = 0;
                sawsugar = 0;
            }*/
        }

        #endregion
    }
}